function [solTx,solSx,Qlty,QltyTx,QltySx]=MinSolKnSx(sxKn,d,angleTx,angleSx,Qsub)
% [solTx,solSx,Qlty,QltyTx,QltySx]=MinSolKnwnSx(sxKn,d,angleTx,angleSx,Qsub)
%
% This routine obtains the minimal measurement solution of a transmitter
% and a scatterer by using one intentional scatterer at a known position. 
%
% Input        
% sxKn:     position vector of intentional scatterers
% d:        TDOA dt and dts measurements
% angleTx:  transmitter AOA measurements
% angleSx:  scatterer AOA measurements
% Qsub:     measurement noise covariance matrix
%
% Output        
% solTx:    transmitter position estimate
% solSx:    scatterer position estimate
% Qlty:     solution quality factor
% QltyTx:   transmitter solution quality factor
% QltySx:   scatterer solution quality factor
%
% The routine can be used for 2D(K=2) or 3D(K=3) localization.
%
% Reference:
% Reference: Y. Zhang and K. C. Ho, "Localization of transmitters and scatterers by single 
% receiver", IEEE Trans. Signal Process., vol. 71, pp. 2267-2282, 2023.
%
% Yang Zhang, K. C. Ho     06-28-2023
%
%       Copyright (C) 2023
%       Computational Intelligence Signal Processing Laboratory
%       University of Missouri
%       Columbia, MO 65211, USA.
%       hod@missouri.edu
%
            
K=size(sxKn,1);
theta1Tx=angleTx(1);
theta1Sx=angleSx(1);

if K==2
    TriSx=[cos(theta1Sx);sin(theta1Sx)];
    TriTx=[cos(theta1Tx);sin(theta1Tx)];
else
    thetaTx=angleTx(2);
    thetaSx=angleSx(2);
    TriSx=[cos(thetaSx)*cos(theta1Sx);cos(thetaSx)*sin(theta1Sx);sin(thetaSx)];
    TriTx=[cos(thetaTx)*cos(theta1Tx);cos(thetaTx)*sin(theta1Tx);sin(thetaTx)];
end

d_em1=(d(1,1)^2-2*d(1,1)*norm(sxKn))/(-2*(d(1,1)-norm(sxKn)+sxKn'*TriTx));
solTx=d_em1*TriTx;

d_sxKn2=(2*d(2,1)*d_em1+d(2,1)^2)/(2*(d(2,1)+d_em1-solTx'*TriSx));
solSx=d_sxKn2*TriSx;

rho_d=[((solTx-sxKn)/norm(solTx-sxKn)-solTx/norm(solTx))',zeros(1,K);
    ((solTx-solSx)/norm(solTx-solSx)-solTx/norm(solTx))',((solSx-solTx)/norm(solTx-solSx)+solSx/norm(solSx))'];
rho_angleTx=[[-solTx(2),solTx(1),zeros(1,K-2)]./((norm([solTx(2),solTx(1)]))^2),zeros(1,K)];
rho_angleSx=[zeros(1,K),[-solSx(2),solSx(1),zeros(1,K-2)]./((norm([solSx(2),solSx(1)]))^2)];
if K==3
    rho_angleTx=[rho_angleTx;[-sin(thetaTx)*cos(theta1Tx),-sin(thetaTx)*sin(theta1Tx),cos(thetaTx)]/d_em1,zeros(1,K)];
    rho_angleSx=[rho_angleSx;zeros(1,K),[-sin(thetaSx)*cos(theta1Sx),-sin(thetaSx)*sin(theta1Sx),cos(thetaSx)]/d_sxKn2];
end
B_rhoi=[rho_d;rho_angleTx;rho_angleSx];
CovInd=inv(B_rhoi)*Qsub*inv(B_rhoi)';
% FIMInd=B_rhoi'*inv(Qsub)*B_rhoi;

if d_em1<0
    QltyTx=Inf;
else
    QltyTx=trace(CovInd(1:K,1:K));
end

if d_sxKn2<0 || d_em1<0
    QltySx=Inf;
else
    QltySx=trace(CovInd(K+1:2*K,K+1:2*K));
end

if d_sxKn2<0 || d_em1<0
    Qlty=Inf;
else
    Qlty=trace(CovInd);
end





