function [gamma,Qlty]=MinSolUnTxUnSx(d,angleTx,angleSx,Qsub)
% [gamma,Qlty]=MinSolUnTxUnSx(d,angleTx,angleSx,Qsub)
%
% This routine obtains the minimal measurement solution of two unknown
% transmitters and two unknown scatterers by using the measurements among them. 
%
% Input       
% d:        TDOA dts measurements
% angleTx:  transmitter AOA measurements
% angleSx:  scatterer AOA measurements
% Qsub:     measurement noise covariance matrix
%
% Output        
% gamma:    transmitter and scatterer position estimates
% Qlty:     solution quality factor
%
% The routine can be used for 2D(K=2) or 3D(K=3) localization.
%
% Reference:
% Reference: Y. Zhang and K. C. Ho, "Localization of transmitters and scatterers by single 
% receiver", IEEE Trans. Signal Process., vol. 71, pp. 2267-2282, 2023.
%
% Yang Zhang, K. C. Ho     06-28-2023
%
%       Copyright (C) 2023
%       Computational Intelligence Signal Processing Laboratory
%       University of MissouriTheta
%       Columbia, MO 65211, USA.
%       hod@missouri.edu
%
            

K=size(angleTx,1)+1;
alpha=angleTx(1,:);
beta=angleSx(1,:);

if K==2
    TriSx1=[cos(beta(1));sin(beta(1))];
    TriSx2=[cos(beta(2));sin(beta(2))];
    TriTx1=[cos(alpha(1));sin(alpha(1))];
    TriTx2=[cos(alpha(2));sin(alpha(2))];
    angCom11=cos(alpha(1)-beta(1));
    angCom12=cos(alpha(1)-beta(2));
    angCom21=cos(alpha(2)-beta(1));
    angCom22=cos(alpha(2)-beta(2));
else
    thetaTx=angleTx(2,:);
    thetaSx=angleSx(2,:);
    TriSx1=[cos(thetaSx(1))*cos(beta(1));cos(thetaSx(1))*sin(beta(1));sin(thetaSx(1))];
    TriSx2=[cos(thetaSx(2))*cos(beta(2));cos(thetaSx(2))*sin(beta(2));sin(thetaSx(2))];
    TriTx1=[cos(thetaTx(1))*cos(alpha(1));cos(thetaTx(1))*sin(alpha(1));sin(thetaTx(1))];
    TriTx2=[cos(thetaTx(2))*cos(alpha(2));cos(thetaTx(2))*sin(alpha(2));sin(thetaTx(2))];
    angCom11=cos(thetaTx(1))*cos(thetaSx(1))*cos(alpha(1)-beta(1))+sin(thetaTx(1))*sin(thetaSx(1));
    angCom12=cos(thetaTx(1))*cos(thetaSx(2))*cos(alpha(1)-beta(2))+sin(thetaTx(1))*sin(thetaSx(2));
    angCom21=cos(thetaTx(2))*cos(thetaSx(1))*cos(alpha(2)-beta(1))+sin(thetaTx(2))*sin(thetaSx(1));
    angCom22=cos(thetaTx(2))*cos(thetaSx(2))*cos(alpha(2)-beta(2))+sin(thetaTx(2))*sin(thetaSx(2));
end

p0=d(2,1)*d(1,1)^2-d(1,1)*d(2,1)^2;
q0=d(2,2)*d(1,2)^2-d(1,2)*d(2,2)^2;

p1=d(2,1)^2*(1-angCom11)-2*d(1,1)*d(2,1);
q1=d(2,2)^2*(1-angCom12)-2*d(1,2)*d(2,2);

p2=d(1,1)^2*(1-angCom21)-2*d(1,1)*d(2,1);
q2=d(1,2)^2*(1-angCom22)-2*d(1,2)*d(2,2);

p3=2*(d(2,1)*(1-angCom11)-d(1,1)*(1-angCom21));
q3=2*(d(2,2)*(1-angCom12)-d(1,2)*(1-angCom22));

a=(p0*q3-q0*p3)/(p1*q3-q1*p3);
b=(p2*q3-q2*p3)/(p1*q3-q1*p3);

if p3*b~=0
    
    temp1=(p2-p3*a-p1*b)/(2*p3*b);
    temp2=sqrt((p3*a+p1*b-p2)^2-4*p3*b*(p1*a-p0))/(2*p3*b);
    temp2=real(temp2);
    
    d_Tx21=temp1+temp2;
    d_Tx22=temp1-temp2;
    d_Tx2Cand=[d_Tx21;d_Tx22];
    
    if d_Tx21>=0 && d_Tx22>=0 
    
        for ii=1:2
            d_Tx2=d_Tx2Cand(ii);
            d_Tx1=a+b*d_Tx2;
            d_Sx1=(d(2,1)^2+2*d(2,1)*d_Tx2)/(2*d(2,1)+2*(1-angCom21)*d_Tx2);
            d_Sx2=(d(2,2)^2+2*d(2,2)*d_Tx2)/(2*d(2,2)+2*(1-angCom22)*d_Tx2);
            
            Tx1Cand(1:K,ii)=d_Tx1*TriTx1;
            Tx2Cand(1:K,ii)=d_Tx2*TriTx2;
            Sx1Cand(1:K,ii)=d_Sx1*TriSx1;
            Sx2Cand(1:K,ii)=d_Sx2*TriSx2;
            Tx=[Tx1Cand(:,ii),Tx2Cand(:,ii)];
            Sx=[Sx1Cand(:,ii),Sx2Cand(:,ii)];
            
            for i=1:2
                for j=1:2
                    dRec(i,j)=norm(Tx(:,i)-Sx(:,j))+norm(Sx(:,j))-norm(Tx(:,i));
                    angleSxRec(1,j)=atan2(Sx(2,j),Sx(1,j));
                    if K==3
                        angleSxRec(2,j)=asin(Sx(3,j)/norm(Sx(:,j)));
                    end
                end
                angleTxRec(1,i)=atan2(Tx(2,i),Tx(1,i));
                if K==3
                    angleTxRec(2,i)=asin(Tx(3,i)/norm(Tx(:,i)));
                end
            end
            Res(ii)=norm(reshape(angleTx,1,[])-reshape(angleTxRec,1,[]))+norm(reshape(angleSx,1,[])-reshape(angleSxRec,1,[]))+norm(reshape(d,[],1)-reshape(dRec,[],1));
        end
        [~,idx]=min(Res);
        Tx1=Tx1Cand(:,idx);
        Tx2=Tx2Cand(:,idx);
        Sx1=Sx1Cand(:,idx);
        Sx2=Sx2Cand(:,idx);
        
    elseif d_Tx21>=0 && d_Tx22<0
        d_Tx2=d_Tx21;
        d_Tx1=a+b*d_Tx2;
        d_Sx1=(d(2,1)^2+2*d(2,1)*d_Tx2)/(2*d(2,1)+2*(1-angCom21)*d_Tx2);
        d_Sx2=(d(2,2)^2+2*d(2,2)*d_Tx2)/(2*d(2,2)+2*(1-angCom22)*d_Tx2);
        
        Tx1(1:K,:)=d_Tx1*TriTx1;
        Tx2(1:K,:)=d_Tx2*TriTx2;
        Sx1(1:K,:)=d_Sx1*TriSx1;
        Sx2(1:K,:)=d_Sx2*TriSx2;
    else
        d_Tx2=d_Tx22;
        d_Tx1=a+b*d_Tx2;
        d_Sx1=(d(2,1)^2+2*d(2,1)*d_Tx2)/(2*d(2,1)+2*(1-angCom21)*d_Tx2);
        d_Sx2=(d(2,2)^2+2*d(2,2)*d_Tx2)/(2*d(2,2)+2*(1-angCom22)*d_Tx2);
        
        Tx1(1:K,:)=d_Tx1*TriTx1;
        Tx2(1:K,:)=d_Tx2*TriTx2;
        Sx1(1:K,:)=d_Sx1*TriSx1;
        Sx2(1:K,:)=d_Sx2*TriSx2;
    end
    
else
    
    d_Tx2=(p0-p1*a)/(p3*a+p1*b-p2);
    d_Tx1=a+b*d_Tx2;
    d_Sx1=(d(2,1)^2+2*d(2,1)*d_Tx2)/(2*d(2,1)+2*(1-angCom21)*d_Tx2);
    d_Sx2=(d(2,2)^2+2*d(2,2)*d_Tx2)/(2*d(2,2)+2*(1-angCom22)*d_Tx2);
    
    Tx1(1:K,:)=d_Tx1*[cos(alpha(1));sin(alpha(1))];
    Tx2(1:K,:)=d_Tx2*[cos(alpha(2));sin(alpha(2))];
    Sx1(1:K,:)=d_Sx1*[cos(beta(1));sin(beta(1))];
    Sx2(1:K,:)=d_Sx2*[cos(beta(2));sin(beta(2))];
end

rho_d=[((Tx1-Sx1)/norm(Tx1-Sx1)-Tx1/norm(Tx1))',zeros(1,K),((Sx1-Tx1)/norm(Tx1-Sx1)+Sx1/norm(Sx1))',zeros(1,K);
    ((Tx1-Sx2)/norm(Tx1-Sx2)-Tx1/norm(Tx1))',zeros(1,K),zeros(1,K),((Sx2-Tx1)/norm(Tx1-Sx2)+Sx2/norm(Sx2))';
    zeros(1,K),((Tx2-Sx1)/norm(Tx2-Sx1)-Tx2/norm(Tx2))',((Sx1-Tx2)/norm(Tx2-Sx1)+Sx1/norm(Sx1))',zeros(1,K);
    zeros(1,K),((Tx2-Sx2)/norm(Tx2-Sx2)-Tx2/norm(Tx2))',zeros(1,K),((Sx2-Tx2)/norm(Tx2-Sx2)+Sx2/norm(Sx2))'];
rho_angleTx=[[-Tx1(2),Tx1(1),zeros(1,K-2)]./(norm([Tx1(2),Tx1(1)])^2),zeros(1,K),zeros(1,2*K);
    zeros(1,K),[-Tx2(2),Tx2(1),zeros(1,K-2)]./(norm([Tx2(2),Tx2(1)])^2),zeros(1,2*K)];
rho_angleSx=[zeros(1,2*K),[-Sx1(2),Sx1(1),zeros(1,K-2)]./(norm([Sx1(2),Sx1(1)])^2),zeros(1,K);
    zeros(1,2*K),zeros(1,K),[-Sx2(2),Sx2(1),zeros(1,K-2)]./(norm([Sx2(2),Sx2(1)])^2)];
if K==3
    rho_angleTx2=[[-sin(thetaTx(1))*cos(alpha(1)),-sin(thetaTx(1))*sin(alpha(1)),cos(thetaTx(1))]/norm(Tx1),zeros(1,K),zeros(1,2*K);
        zeros(1,K),[-sin(thetaTx(2))*cos(alpha(2)),-sin(thetaTx(2))*sin(alpha(2)),cos(thetaTx(2))]/norm(Tx2),zeros(1,2*K)];
    rho_angleSx2=[zeros(1,2*K),[-sin(thetaSx(1))*cos(beta(1)),-sin(thetaSx(1))*sin(beta(1)),cos(thetaSx(1))]/norm(Sx1),zeros(1,K);
        zeros(1,2*K),zeros(1,K),[-sin(thetaSx(2))*cos(beta(2)),-sin(thetaSx(2))*sin(beta(2)),cos(thetaSx(2))]/norm(Sx2)];
    
    rho_angleTx=[rho_angleTx;rho_angleTx2];
    rho_angleSx=[rho_angleSx;rho_angleSx2];
end

B_rhoi=[rho_d;rho_angleTx;rho_angleSx];
CovInd=inv(B_rhoi)*Qsub*inv(B_rhoi)';
% FIMInd=B_rhoi'*inv(Qsub)*B_rhoi;
% Qlty=det(FIMInd);
Qlty=trace(CovInd);

gamma=[Tx1;Tx2;Sx1;Sx2];






